/* This file is part of dc_qt.

    dc_qt is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    dc_qt is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dc_qt; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "dc_qt.h"
#include "connect_dlg.h"
#include "dctc_hub.h"
#include "user_lv_item.h"
#include "dc_parse.h"
#include "user_lv.h"
#include "file_list_dialog.h"
#include "preferences_dlg.h"
#include "file_transfer_lv.h"
#include "dc_settings.h"
#include "dc_hub.h"
#include "search_dlg.h"

#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qmenubar.h>
#include <qlistbox.h>
#include <qsplitter.h>
#include <qlistview.h>
#include <qinputdialog.h>
#include <qlayout.h>
#include <qvbox.h>

#include <unistd.h>

#include <algorithm>

dc_main_window::dc_main_window(int argc, char *argv[]) : QWidget(0, "dc main window")
{
	for (int i = 0; i < argc; i++)
		cmd_line.append(argv[i]);
	
	hubs = new dc_hub();

	init_users = false;
	
	cur_hub = 0;
	search_res_dlg = 0;
	
	resize(800, 600);
	QPopupMenu *file = new QPopupMenu(this, "file menu");

	file->insertItem("&New Instance", this, SLOT(new_instance()));
	file->insertSeparator();
	file->insertItem("&Connect", this, SLOT(dc_connect()));
	file->insertItem("&Disconnect", this, SLOT(dc_disconnect()));
	file->insertSeparator();
	file->insertItem("&Search", this, SLOT(search()));
#ifdef _DEBUG
	file->insertItem("S&end Command", this, SLOT(send_command()));
	file->insertSeparator();
#endif
	file->insertItem("&Preferences",this,SLOT(preferences()));
	file->insertSeparator();
	file->insertItem("&Quit", this, SIGNAL(quit()));
	file->insertItem("Disconnect && Quit", this, SLOT(disconnect_quit()));

	QMenuBar *menu = new QMenuBar(this, "main menu");
	menu->insertItem("&File", file);

	// in the top row there is a menu, in the middle there is a message
	// pane and the user list, in the bottom there is a listbox that is
	// currently unused, but will display the transfer queue
	QGridLayout *layout = new QGridLayout(this, 3, 1);
	layout->addWidget(menu, 0, 0);
	layout->setRowStretch(0, 0);
	layout->setRowStretch(1, 4);
	layout->setRowStretch(2, 1);

	// this is called vert, because I like to think of the direction of the
	// splitter, not the arrangements of the things around the splitter
	vert = new QSplitter(QSplitter::Horizontal, this, "vert splitter");
	vert->setOpaqueResize();
	layout->addWidget(vert, 1, 0);

	// this doesn't appear for some reason
	horiz = new QSplitter(QSplitter::Vertical, this, "hor splitter");
	layout->addWidget(horiz, 2, 0);

	QVBox *msgBox = new QVBox(vert);
	message_view = new QListBox(msgBox, "message_view");
	message_view->setMinimumWidth(100);

	chat_edit = new QLineEdit(msgBox, "chat edit");

	user_list = new user_lv(vert, "user_list");
	user_list->setup_columns();
	user_list->setMinimumWidth(100);

	file_transfers = new file_transfer_lv(horiz, "file transfers");
	file_transfers->setMinimumHeight(100);
}

void dc_main_window::dc_connect()
{
	connect_dlg *dlg = new connect_dlg(hubs, this, "connect_dlg", true);
	if (dlg->exec() == QDialog::Accepted)
	{
		// make a new dctc_hub interface and push in onto the list
		// of active hubs
		hub_info hub = dlg->selected_hub();
		cur_hub = new dctc_hub(0, "hub-" + hub.ip);

		if (hub.ip == QString::null)
			return;

		int ret;
		if (dlg->selected_running())
			ret = cur_hub->hubconnect(hub.ip, hub.pid);
		else
			ret = cur_hub->hubconnect(hub.ip);

		if (ret != CONNECT_SUCCESS)
		{
			QMessageBox::critical(this, "Connect", "Could not connect to hub");
			delete cur_hub;
			cur_hub = 0;
			delete dlg;
			return;
		}

		active_hubs.push_back(cur_hub);
		
		// connect all the signals
		// probably should be moved into a function
		connect(cur_hub, SIGNAL(hub_died()), SLOT(dc_disconnect()));
		connect(cur_hub, SIGNAL(user_added(const QString &, bool)), SLOT(add_user(const QString &, bool)));
		connect(cur_hub, SIGNAL(user_removed(const QString &)), SLOT(rm_user(const QString &)));
		connect(cur_hub, SIGNAL(user_info(const QString&, const QString&, const QString&, const QString&)),
				SLOT(update_user(const QString&, const QString&, const QString&, const QString&)));
		connect(user_list, SIGNAL(get_user_list(const QString&)), cur_hub, 
				SLOT(get_user_list(const QString&)));
		connect(cur_hub, SIGNAL(logging(const QString &)), SLOT(receive_log(const QString &)));
		connect(cur_hub, SIGNAL(user_flist_file(const QString &, const QString &, const QString &)), 
				SLOT(user_flist_file(const QString&, const QString &, const QString &)));
		connect(cur_hub, SIGNAL(search_file(const QString &, const QString &, const QString&, 
						const QString&)), SLOT(search_file(const QString &, const QString &,
							const QString &, const QString &)));
		connect(cur_hub, SIGNAL(new_transfer(int, const QString &, const QString &, const QString &)), 
				file_transfers,	SLOT(add_transfer(int, const QString &, const QString &, const QString &)));
		connect(cur_hub, SIGNAL(done_transfer(const QString &, const QString &)), 
				file_transfers, SLOT(remove_transfer(const QString &, const QString&)));
		connect(cur_hub, SIGNAL(done_transfer(int)), file_transfers, SLOT(remove_transfer(int)));
		connect(cur_hub, SIGNAL(chat_msg(const QString &)), SLOT(receive_chatmsg(const QString &)));
		connect(chat_edit, SIGNAL(returnPressed()), SLOT(send_chatmsg()));
		connect(file_transfers, SIGNAL(cancel_transfer(int)), cur_hub, SLOT(cancel_transfer(int)));
		connect(file_transfers, SIGNAL(get_flist(const QString &)), 
				cur_hub, SLOT(get_user_list(const QString &)));

		vlog("Connected to: " + hub.ip);

//		message_view->insertItem(cur_hub->message);
		settings.save_favourites_file();   // To save the new favourites if any
	}
	else
	{
	}

	delete dlg;
}

void dc_main_window::dc_disconnect()
{
	vlog("cur_hub");
	vlog(cur_hub);
	if (cur_hub)
	{
		vlog("Disconnecting from: " + cur_hub->get_hubip());
	
		cur_hub->hubdisconnect();
		disconnect(cur_hub, SLOT(cancel_transfer(int)));
		disconnect(cur_hub, SLOT(get_user_list(const QString &)));
	}

	// disconnect all the signals, should be synced with the connects above
	disconnect(this, SLOT(dc_disconnect()));
	disconnect(this, SLOT(update_user(const QString&, const QString&, const QString&, const QString&)));
	disconnect(this, SLOT(add_user(const QString &, bool)));
	disconnect(this, SLOT(rm_user(const QString &)));
	disconnect(this, SLOT(receive_log(const QString &)));
	disconnect(this, SLOT(user_flist_file(const QString &, const QString &, const QString &)));
	disconnect(this, SLOT(search_file(const QString &, const QString &, const QString&, const QString&)));
	if (file_transfers)
	{
		disconnect(file_transfers, SLOT(add_transfer(int, const QString &, const QString &, 
						const QString &)));
		disconnect(file_transfers, SLOT(remove_transfer(const QString &, const QString &)));
		disconnect(file_transfers, SLOT(remove_transfer(int)));
	}
	
	// delete the hub
	active_hubs.remove(cur_hub);
	clear_user_list();

	// only display the message if we were connected
	if (cur_hub)
		QMessageBox::information(this, "Disconnected", "Disconnected from hub");

	delete cur_hub;
	cur_hub = 0;
}

void dc_main_window::disconnect_quit()
{
	dc_disconnect();
	emit quit();
}

void dc_main_window::update_user(const QString &user, const QString &shared, const QString &con, 
		const QString &desc)
{
	// find a user and update with user list info
	list<user_lv_item*>::iterator u = find_user(user);

	if (u == displayed_users.end())
		return;

	(*u)->connection(con);
	(*u)->shared(shared);
	(*u)->description(desc);
}

void dc_main_window::rm_user(const QString &user)
// pre: the leading quotes have not already been striped from user
{
	dc_parse p(user, '|');
	p.remove_quotes();

	list<user_lv_item*>::iterator del = find_user(p.get_str(0));

	displayed_users.erase(del);

	delete *del;
}

// return an iterator to user in the displayed_users list
// crappy error handling right now
list<user_lv_item*>::iterator dc_main_window::find_user(const QString& user)
{
	list<user_lv_item*>::iterator itr = ::find(displayed_users.begin(), displayed_users.end(), user);
	
	if (itr == displayed_users.end())
	{
		vlog("find_user: user " + user + " not found");
		vlog(cur_hub->message);
		return displayed_users.end();
	}

	return itr;
}
	
void dc_main_window::add_user(const QString &user, bool ulist)
// pre: the leading quotes have not already been striped from user
{
	dc_parse p(user, '|');
	p.remove_quotes();

	QString u = p.get_str(0);
	
	if (::find(displayed_users.begin(), displayed_users.end(), u) != displayed_users.end())
		return;
	
//	cur_hub->log("added user: " + u);

	if (ulist)
		cur_hub->get_uinfo(u);	

	displayed_users.push_back(new user_lv_item(user_list, u));
}

#ifdef _DEBUG
void dc_main_window::send_command()
{
	bool ok;
	
	QString text = QInputDialog::getText("Send Command", "Enter Command", QLineEdit::Normal, 
				QString::null, &ok, this);

	if (ok)
		cur_hub->send_command(text);
}
#endif

void dc_main_window::clear_user_list()
{
	while (displayed_users.size() > 0)
	{
		user_lv_item* l = displayed_users.front();
		displayed_users.pop_front();
		delete l;
	}
}

void dc_main_window::receive_log(const QString &l)
{
	return;
	
	static long i = 0;
	static const int MAX_SIZE = 100;
	
	message_view->insertItem(l, i);

	if (message_view->count() > MAX_SIZE)
	{
		message_view->removeItem(i - MAX_SIZE);
	}
}

void dc_main_window::user_flist_file(const QString &user, const QString &file, const QString &size)
{
	// find the selected user and open a dialog with the users files
	list<file_list_dialog*>::iterator loc = ::find(file_list_dlgs.begin(), file_list_dlgs.end(), user);

	if (loc == file_list_dlgs.end())
	{
		file_list_dialog *d = new file_list_dialog (user);
		file_list_dlgs.push_back(d);
		loc = ::find(file_list_dlgs.begin(), file_list_dlgs.end(), user);

		// for file transfers
		connect (d, SIGNAL (get_file (const QString &, const QString &)), cur_hub, 
				SLOT (get_file (const QString &, const QString &)));
	}

	(*loc)->setCaption(user);
	(*loc)->resize(400, 300);
	(*loc)->show();
	(*loc)->add_file(file, size);
}

// only one search window for now
void dc_main_window::search_file(const QString &user, const QString &file, const QString &size, 
		const QString &slot)
{
	if (!search_res_dlg)
	{
		return;
	}
	else
	{
		if (search_res_dlg->isVisible() == false)
		{
			search_res_dlg->clear_files();
			search_res_dlg->show();
		}
	}
	
	search_res_dlg->add_file(user, file, size, slot);
}

// TODO: file type
void dc_main_window::search()
{
	if (!cur_hub)
		return;
	
	search_dlg *dlg = new search_dlg(this);

	if (dlg->exec() == search_dlg::Accepted)
	{
		cur_hub->search_for(dlg->get_search_pattern(), dlg->get_search_type());

		delete search_res_dlg;
		search_res_dlg = new search_list_dialog("1", dlg->get_search_pattern());
		search_res_dlg->setCaption("Search Results");
		search_res_dlg->resize(400, 300);
		search_res_dlg->show();

		connect (search_res_dlg, SIGNAL(get_file(const QString &, const QString &)), cur_hub,
				SLOT (get_file( const QString &, const QString &)));
		connect (search_res_dlg, SIGNAL(get_flist(const QString &)), cur_hub, 
				SLOT (get_user_list(const QString &)));
	}
}

void dc_main_window::preferences()
{
	preferences_dlg *pref = new preferences_dlg();
	pref->show();
}

void dc_main_window::send_chatmsg()
{
	cur_hub->send_chatmsg(chat_edit->text());
	chat_edit->clear();
}

void dc_main_window::receive_chatmsg(const QString &msg)
{
	message_view->insertItem(msg);
}

void dc_main_window::new_instance()
{
	int f = fork();

	const char* argv[cmd_line.count() + 1];

	for (int i = 0; i < cmd_line.count(); i++)
		argv[i] = cmd_line[i].latin1();

	argv[cmd_line.count()] = 0;
	
	if (f == 0)
	{
		int rc = execvp(argv[0], (char * const *)argv);
		cout << rc << endl;
		_exit(-1);
	}
}
