/* This file is part of dc_qt.

    dc_qt is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    dc_qt is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dc_qt; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "dctc_connection.h"
#include "util.h"
#include "dc_settings.h"

#include <string>

#include <qsocketnotifier.h>

#include <unistd.h>
#include <signal.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/un.h>
#include <errno.h>
#include <stdio.h>

const int MAX_WAIT = 180;

dctc_connection::dctc_connection(QObject *parent, const char *name)
	: QObject(parent, name)
{
	pid = 0;
}

void dctc_connection::kill_client()
{
	pid = 0;
	
/*
 * The preferred method of killing dctc is by sending
 * /FORCEQUIT
 * so all this crap is no longer necessary
 *
	if (pid == 0)
		return;

	QString debug;
	debug.sprintf("killing pid: %d", pid);
	log(debug);
	kill(pid, SIGQUIT);

	// if the client doesn't respond to a quit, repeatedly try to kill it
	while (waitpid(pid, NULL, WNOHANG) == -1)
	{
		usleep( (1000000/180)*20);
		kill(pid, SIGKILL);
	}
	
	pid = 0;
*/
}

#define GET_SETTING(x, y) \
if (settings.get_setting(x) != QString::null) \
{ \
	result[i++] = y; \
	result[i++] = settings.get_setting(x); \
}

// fills result with the command line parameters that the text
// client should be started with
void dctc_connection::get_flags(const char **result, const char *hubip)
{
	int i = 0;
	result[i++] = "dctc";
	result[i++] = "-l";
	result[i++] = "-t";
	result[i++] = "-g";
	result[i++] = hubip;
	GET_SETTING(dc_settings::NICKNAME, "-n");
	GET_SETTING(dc_settings::DESCRIPTION, "-i");
	GET_SETTING(dc_settings::SLOTS, "-d");
	GET_SETTING(dc_settings::OFFSET, "-o");
	GET_SETTING(dc_settings::ADDRESS, "-a");
	GET_SETTING(dc_settings::PORT, "-p");
	GET_SETTING(dc_settings::CONNECTION, "-c");
	result[i++] = "--precmd";
	result[i++] = "/RECOND 30";
	result[i++] = "--precmd";
	result[i++] = "/REBUILD 900";

	for (int j = 0; j < settings.num_shares(); j++)
	{
		result[i++] = "-s";
		result[i++] = settings.get_share(j);
	}

	if (settings.get_setting(dc_settings::PASSIVE) == "true")
		result[i++] = "-f";
}

// handles the forking of the client to start the dctc text client
// this crashes a lot, I have no idea why.  Maybe sleep_on_client isn't
// working properly
int dctc_connection::fork_client(const char *hubip)
{
	// f == 0 if this is the forked process
	// f == pid of child otherwise
	int f = fork();

	// this is the forked process
	if (f == 0)
	{
		// we need an array of char* to pass to the new process
		// 32 should be enough.  I need a better way to do this

		const char *args[64];

		// clear out all the flags in case some are not used
		for (int i = 0; i < 64; i++)
			args[i] = "";
		
		get_flags(&args[0], hubip);

		QString debug;
		
		for (int i =0; i < 64; i++)
		{
			debug += args[i] + QString(" ");
		}

		vlog("cmd line: " + debug);

		// execute the dctc client
		int rc = execvp("dctc", (char *const *)args);

		vlog("rc: " + rc);

		// we shouldn't reach this
		_exit(-1);
	}

	// this is the parent process
	
	// we use the pid later to kill the correct dctc client
	pid = f;

	return connect_to_running(hubip);
}

int dctc_connection::connect_to_running(const char *hubip, int new_pid = 0)
{	
	vlog(QString("Connecting to ") + hubip);
	QString debug;
	debug.sprintf("pid = %d", new_pid);
	vlog(debug);

	if (new_pid)
		pid = new_pid;

	dctc_fd = socket(AF_UNIX, SOCK_STREAM, 0);

	if (dctc_fd == -1)
	{
		errlog("Could not create socket\n");
		return COULD_NOT_CREATE_SOCKET;
	}

	struct sockaddr_un name;
	name.sun_family = AF_UNIX;

	char pidhex[9];
	sprintf(pidhex, "%08X", pid);
	
	// we open the socket created by dctc in $HOME/.dctc/running/dctc-xxxxx-yyyyy
	// where xxxxx == the pid of the client in hex
	// yyyyy == the ip of the hub
	QString socket_path = settings.get_home_dir() + "/.dctc/running/";
	socket_path += "dctc-";
	socket_path += pidhex;
	socket_path += "-";
	socket_path += hubip;
	strcpy(name.sun_path, socket_path.latin1());
	
	// must sleep so that dctc has time to get setup
	if (sleep_on_client(socket_path.latin1()) == false)
	{
		kill_client();
		return SLEEP_TIMED_OUT;
	}
	
	int retcode = ::connect(dctc_fd, (sockaddr*)&name, sizeof(struct sockaddr_un));
	if (retcode == -1)
	{
		errlog("Could not connect to socket " + socket_path);
		errlog("retcode: " + retcode);
		errlog("dctc_fd: " + dctc_fd);

		kill_client();
		return COULD_NOT_CONNECT_TO_SOCKET;
	}
	
	// the notifier calls dataReceived whenever the socket has something to read
	read_notifier = new QSocketNotifier(dctc_fd, QSocketNotifier::Read);
	QObject::connect(read_notifier, SIGNAL(activated(int)), this, SLOT(dataReceived(int)));

	return CONNECT_SUCCESS;
}

// handler for receiving data through the socket
void dctc_connection::dataReceived(int)
{
	// if there is no running dctc, then we can't receive anything
	if (!pid)
	{
		QObject::disconnect(this, SLOT(dataReceived(int)));
		return;
	}
	
	int n;
	char c;

	message = "";

	// we don't know how many characters we're going to read, so read one
	// at a time until we reach a \n
	do
	{
		n = read(dctc_fd, &c, 1);

		// if we didn't read one character, we've got an error
		if (n != 1)
		{
			cerr << "error in dataReceived\n";
			kill_client();
			return;
		}

		message += c;
	} while (c != '\n');

	// deal with the message that we got back
	emit dctc_data(message);
}

// send the string command to the text client
// the required newline is added by the function
void dctc_connection::send_command(const QString &command)
{
	vlog("send_command(" + command + ");");

	QString cmd(command + "\n");
	
	int ret = ::send(dctc_fd, cmd.latin1(), cmd.length(), MSG_DONTWAIT);

	// everything ok
	if (ret == cmd.length())
		return;

	if (errno != EAGAIN)
	{
		cerr << "error sending!  ret: " << ret << " errno: " << errno << endl;
		kill_client();
	}
}

bool dctc_connection::sleep_on_client(const QString& socket)
{
	struct stat st;

	for (int i = 0; i < MAX_WAIT; i++)
	{
		if (stat(socket.latin1(), &st) == 0)
			return true;

		if (waitpid(pid, NULL, WNOHANG) == -1)
			return false;

		usleep( (1000000/180)*20);
	}

	return false;
}

int dctc_connection::hubconnect(const QString &hubip, int new_pid = 0)
{
	// we can reconnect to this dctc later, so don't disconnect
//	if (pid)
//		hubdisconnect();

	if (new_pid)
		return connect_to_running(hubip.latin1(), new_pid);
	
	// handle the starting of the dctc text client
	int ret = fork_client(hubip.latin1());
	if (ret != CONNECT_SUCCESS)
	{
		errlog("Could not connect");
		kill_client();
	}
	
	return ret;
	
	// retrieve the userlist
//	send_command("/ULIST");
}

// disconnect from this hub and kill the associated dctc client
void dctc_connection::hubdisconnect()
{
	send_command("/FORCEQUIT");
	pid = 0;
}

void dctc_connection::data_to_send(const QString &cmd)
{
	send_command(cmd);
}
