/* This file is part of dc_qt.

    dc_qt is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    dc_qt is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dc_qt; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "dctc_hub.h"
#include "dc_parse.h"
#include "dctc_connection.h"
#include "file_transfer.h"
#include "util.h"
#include "dc_settings.h"

#include <procbuf.h>
#include <qstring.h>
#include <string>
#include <sys/types.h>

const char* USER_CMD = "USER ";
const char* OP_CMD = "OP   ";
const char* USER_PLUS_CMD = "USER+";
const char* USER_MINUS_CMD = "USER-";
const char* USER_INFO_CMD = "UINFO";
const char* HUB_MINUS_CMD = "HUB- ";
const char* FLST_CMD = "FLST ";
const char* ERR_CMD = "ERR  ";
const char* DL_START_CMD = "$DL+ ";
const char* DL_DONE_CMD = "$DL- ";
const char* DL_RUN_CMD = "$DL= ";
const char* DL_ABORT_CMD = "$DL# ";
const char* DL_QUEUE_CMD = "$DL~ ";
const char* SEARCH_RESULT = "SREST";
const char* CHAT_CMD = "CHAT ";
const char* XFERR_CMD = "XFERR";

dctc_hub::dctc_hub(QObject *parent = 0, const char* name = 0) 
	: QObject(parent, name)
{
	connection = new dctc_connection(parent, QString("conn-") + name);
}

dctc_hub::~dctc_hub()
{
	delete connection;
	connection = 0;
}

int dctc_hub::hubconnect(const QString &ip, int pid = 0) 
{
	int ret = connection->hubconnect(ip, pid);
	if (ret != CONNECT_SUCCESS)
	{
		errlog("Could not connect to hub");
		return ret;
	}

	if (pid)
	{
		get_ulist();
		send_command("/XFER");
	}

	QObject::connect(connection, SIGNAL(dctc_data(const QString&)), SLOT(dc_string(const QString &)));
//	QObject::connect(this, SIGNAL(hub_died()), connection, SLOT(hub_died()));

	hubip = ip;

	return CONNECT_SUCCESS;
}

void dctc_hub::hubdisconnect()
{
	if (!connection)
		return;
	
	connection->hubdisconnect();
	QObject::disconnect(this, SLOT(dc_string(const QString &)));
}

void dctc_hub::search_for(const QString &pattern, int type = 1)
{
	QString cmd;
	cmd.sprintf("=%s=%d", pattern.latin1(), type);
	send_command("/SRCH " + cmd);
}

void dctc_hub::download_file(const dc_user &from_user, const QString &file)
{
}

// given a string received from the text client, emit the right signal
// so that the window can be updated correctly
void dctc_hub::parse_dctc_string(const QString &str)
{
	vlog(str);
	
	string debug(str.latin1());
	
	QString command_type(dctc_command_type(str));
	
	if (command_type == USER_CMD || command_type == OP_CMD)
		emit user_added(dctc_command_data(str), true);
	else if (command_type == USER_PLUS_CMD)
		emit user_added(dctc_command_data(str));
	else if (command_type == USER_MINUS_CMD)
		emit user_removed(dctc_command_data(str));
	else if (command_type == HUB_MINUS_CMD)
	{
		vlog("emit hub_died()");
		emit hub_died();
	}
	else if (command_type == USER_INFO_CMD)
		parse_uinfo(dctc_command_data(str));
	else if (command_type == FLST_CMD)
	{
		QString file, user, size;
		parse_file(dctc_command_data(str), user, file, size);
		emit user_flist_file(user, file, size);
	}
	else if (command_type == SEARCH_RESULT)
	{
		QString file, user, size, slot;
		parse_srest(dctc_command_data(str), user, file, size, slot);
		emit search_file(user, file, size, slot);
	}
	else if (command_type == DL_RUN_CMD || command_type == XFERR_CMD)
		parse_xferr(str);
	else if (command_type == DL_DONE_CMD || command_type == DL_ABORT_CMD)
	{
		dc_parse p(dctc_command_data(str), '|');
		p.remove_quotes();
		emit done_transfer(p.get_str(0), p.get_str(1));
	}
	else if (command_type == CHAT_CMD)
	{
		dc_parse p(dctc_command_data(str), '|');
		p.remove_quotes();
		emit chat_msg(p.get_str(0));
	}
	else if (command_type == ERR_CMD)
//		log("Err: " + dctc_command_data(str));
		;
	else
	{
		vlog("Unknown command: " + str);
		return;
	}
}

void dctc_hub::parse_uinfo(const QString &str)
{
	// UINFO] ""nick
	// that space kills us, so remove it
	// it'll be the first char, since UINFO] has already been chopped
	QString t(str.right(str.length() - 1));

	// q is required, because there is a | at the end of every message
	dc_parse q(t, '|');
	q.remove_quotes();
	dc_parse p(q.get_str(0), ' ');

	// if the desc field is blank, then dc_parse craps out
	QString desc = "";
	if (p.num_strs() >= 6)
		desc = p.get_str(5);

	emit user_info(p.get_str(0), p.get_str(1), p.get_str(2), desc);	
}

// return the text before the ']' in the text client string
QString dctc_hub::dctc_command_type(const QString &str)
{
	QString ret;

	for (int i = 0; i < str.length() && str[i] != ']'; i++)
		ret += str[i];

//	string debug(ret.latin1());
	
	return ret;
}

// return the data after the ']' in the text client string
// may need to be used in conjuction with a dc_parse
QString dctc_hub::dctc_command_data(const QString &str)
{
	QString ret;
	bool data = false;

	for (int i = 0; i < str.length(); i++)
	{
		if (data)
		{
			ret += str[i];
			continue;
		}

		if (!data && str[i] == ']')
		{
			data = true;
		}
	}

//	string debug(ret.latin1());

	return ret;
}

void dctc_hub::send_command(const QString &command)
{
	if (connection)
		connection->send_command(command);
}

void dctc_hub::dc_string(const QString &dc)
{
	parse_dctc_string(dc);
}

void dctc_hub::get_uinfo(const QString &user)
{
	send_command("/UINFO " + user);
}

void dctc_hub::get_user_list(const QString &user)
{
	send_command("/LS " + user);
}

void dctc_hub::parse_file(const QString &str, QString &user, QString &file, QString &size)
{
	dc_parse p(str, '|');
	p.remove_quotes();

	user = p.get_str(0);
	file = p.get_str(1);
	size = p.get_str(2);
}

void dctc_hub::parse_srest(const QString &str, QString &user, QString &file, QString &size, QString &slot)
{
	dc_parse p(str, '|');
	p.remove_quotes();

	user = p.get_str(0);
	file = p.get_str(1);
	size = p.get_str(2);
	slot = p.get_str(3);
}

void dctc_hub::get_file(const QString &user, const QString &file)
{
	QString cmd;
	dc_parse p(file, '\\');
	log("Get file: " + p.get_str(p.num_strs() - 1));
	cmd = "/DL *" + user + "*" + settings.get_setting(dc_settings::DLDIR) + 
		"/" + p.get_str(p.num_strs() - 1) + "*" + file + "*";
	send_command(cmd);
}

void dctc_hub::parse_xferr(const QString &str)
{
	dc_parse p(str, '|');
	p.remove_quotes();

	int transfer_id = p.get_str(0).toInt();
	QString nick = p.get_str(1);

	QString type = p.get_str(2);

	if (type == "Idle")
	{
	}
	else if (type == "LS/")
	{
	}
	// have to do it this way, because of the crazy dctc 
	// socket communications
	// for UL/DL the format is UL/filename or DL/remote filename
	// This means we've got to parse the string again, using '/' as
	// the separator
	else
	{
		QString cmd = dctc_command_type(str);

		dc_parse q(dctc_command_data(str), '/');
		q.remove_quotes();
		
		if (q.get_str(0).contains("UL"))
		{
		}
		else if (q.get_str(0).contains("DL"))
		{
			QString remote_fname = q.get_str(1);
			QString local_fname = p.get_str(3);
			off_t dl_start_pos = p.get_str(4).toLong();
			QString dl_size = p.get_str(5);
			QString start_time = p.get_str(6);
			QString host = p.get_str(7); // this is of the form host:port  UGH!

			emit new_transfer(transfer_id, nick, local_fname, dl_size);
		}
		else if (q.get_str(0).contains("LS"))
		{
		}
		else if (q.get_str(0).contains("Idle"))
		{
		}
		else
		{
			log(q.get_str(0));
			log(str);
			log(cmd);
			assert(false);
		}
	}
}

void dctc_hub::send_chatmsg(const QString &msg)
{
	send_command("/CHAT " + msg);
}

void dctc_hub::cancel_transfer(int id)
{
	QString sid;
	sid.sprintf("%d", id);
	send_command("/KILL " + sid);

	emit done_transfer(id);
}
