/* This file is part of dc_qt.

    dc_qt is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    dc_qt is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dc_qt; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "file_lv_item.h"
#include "util.h"
#include "dc_parse.h"

#include "pixmap.h"

file_lv_item::file_lv_item( directory_lv_item *parent, const QString &s1, const QString &s2 )
	: QListViewItem(parent, s1, bytes_to_units(s2) ), pix( 0 ), dc_file(parent, s1, s2) {}

void file_lv_item::setPixmap( QPixmap *p )
{
    pix = p;
    setup();
    widthChanged( 0 );
    invalidateHeight();
    repaint();
}


const QPixmap *file_lv_item::pixmap( int i ) const
{
    if ( i )
	return 0;
    return pix;
}

#ifdef _QT_3
int file_lv_item::compare(QListViewItem* i, int col, bool ascending) const
{
	// if i has children, then it is a dir and this is greater than it
	if (i->firstChild())
		return 1;

	// if we get here, we know we're comparing to a file
	if (col == 0)
		return text(col).upper().compare(i->text(col).upper());
	if (col == 1)
	{
		double thisbytes = units_to_bytes(text(col));
		double ibytes = units_to_bytes(i->text(col));

		if (thisbytes > ibytes)
			return 1;
		if (thisbytes < ibytes)
			return -1;
		return 0;
	}

	return QListViewItem::compare(i, col, ascending);
}
#endif

/*****************************************************************************
 *
 * Class directory_lv_item
 *
 *****************************************************************************/

directory_lv_item::directory_lv_item( directory_lv_item * parent, const QString& filename )
    : QListViewItem( parent ), 
      pix( 0 ), dc_dir(parent, filename)
{
	setPixmap ( folderClosed );
}

// if the parent is not a directory_lv_item, this is the root dir, so we call
// the dc_dir constructor with parent = 0
directory_lv_item::directory_lv_item( QListView * parent, const QString& filename)
	: QListViewItem(parent), 
	pix(0), dc_dir(filename)
{
	setPixmap(folderClosed); 
}

void directory_lv_item::setPixmap( QPixmap *px )
{
    pix = px;
    setup();
    widthChanged( 0 );
    invalidateHeight();
    repaint();
}

const QPixmap *directory_lv_item::pixmap( int i ) const
{
    if ( i )
	return 0;
    return pix;
}

void directory_lv_item::setOpen( bool o )
{
    if ( o )
		setPixmap( folderOpen );
    else
		setPixmap( folderClosed );

	// if we just opened this dir
    if ( o && !childCount() ) 
	{
		for (int i = 0; i < num_files(); i++)
		{
			dc_file* f = get_file(i);

			if (f->is_dir())
			{
				new directory_lv_item(this, f->get_name());
			}
			else
			{
				new file_lv_item(this, f->get_name(), f->get_size());
			}
		}
    }
	
    QListViewItem::setOpen( o );
}


void directory_lv_item::setup()
{
    setExpandable( TRUE );
    QListViewItem::setup();
}


QString directory_lv_item::full_name() const
{
	return get_full_name();
}


QString directory_lv_item::text( int column ) const
{
    if ( column == 0 )
		return get_name();
	else if (get_parent() == 0)
		return QListViewItem::text(column);
    else 
	{
		QString files;
		files.sprintf("%d files", num_files());
		return files;
	}
}

#ifdef _QT_3
int directory_lv_item::compare(QListViewItem* i, int col, bool ascending) const
{
	// determine if the other item is a dir or not.  If it no children, then it's a file
	// and a directory is greater than it
	if (!i->firstChild())
		return -1;
	
	// if we get here, we're comparing with another directory
	directory_lv_item *d = (directory_lv_item*)i;
	
	if (col == 0)
		return text(col).upper().compare(d->text(col).upper());
	// this is not a root, so compare the number of files in the dir
	else if (col == 1 && get_parent())
	{
		if (num_files() > d->num_files())
			return 1;
		if (num_files() < d->num_files())
			return -1;

		return 0;
	}
	// this is a root, so compare slots
	else if (col == 1 && !get_parent())
	{
		dc_parse pthis(text(col), '/');
		dc_parse pi(i->text(col), '/');

		int numthis = pthis.get_str(0).toInt();
		int denthis = pthis.get_str(1).toInt();
		int numi = pi.get_str(0).toInt();
		int deni = pi.get_str(1).toInt();

		if (numthis > numi)
			return 1;
		if (numthis < numi)
			return -1;
		if (denthis > deni)
			return 1;
		if (denthis < deni)
			return -1;

		return 0;
	}

	return QListViewItem::compare(i, col, ascending);
}
#endif
